==> termilary-0.2.1.tar.gz: lib/src/canvas_terminal.dart <==
import "package:utilary/utilary.dart";
import "glyph.dart";
import "terminal.dart";
import "font.dart";
import "dart:html";

class CanvasTerminal extends Terminal{
  final Array2D<Glyph> _glyphs;
  final Array2D<Glyph> _changedGlyphs;

  final Font _font;
  final CanvasElement _canvas;
  final CanvasRenderingContext2D _context;

  final int _scale = window.devicePixelRatio.toInt();

  factory CanvasTerminal(int width, int height, Font font, [CanvasElement canvas]) {
    if (canvas == null) {
      canvas = CanvasElement();
      document.body.append(canvas);
    }

    return CanvasTerminal._(width, height, font, canvas);
  }

  CanvasTerminal._(int width, int height, this._font, this._canvas)
      : _context = _canvas.getContext("2d"),
        _glyphs = Array2D(height, width),
        _changedGlyphs = Array2D(height, width, Glyph.clear),
        super(width, height) {
    int canvasWidth = cols * _font.charWidth;
    int canvasHeight = rows * _font.lineHeight;

    _canvas.width = canvasWidth * _scale;
    _canvas.height = canvasHeight * _scale;
    _canvas.style.width = "${canvasWidth}px";
    _canvas.style.height = "${canvasHeight}px";

    _context.font = "${_font.size * _scale}px ${_font.family}, monospace";
  }

  Rectangle get bounds {
    var rect = _canvas.getBoundingClientRect();

    return Rectangle(rect.left, rect.top, rect.width, rect.height);
  }

  @override
  void setGlyph(int x, int y, Glyph glyph) {
    if (x < 0) return;
    if (x >= cols) return;
    if (y < 0) return;
    if (y >= rows) return;

    if (_glyphs.get(x, y) != glyph)
      _changedGlyphs.set(x, y, glyph);
    else
      _changedGlyphs.set(x, y, null);
  }

  void _renderGlyph(int x, int y, Glyph glyph) {
    //Fill the background
    _context.fillStyle =
        glyph.back.cssString; //Set context's fill color as glyph's backcolor
    _context.fillRect(
        x * _font.charWidth * _scale,
        y * _font.lineHeight * _scale,
        _font.charWidth * _scale,
        _font.lineHeight * _scale); //Draw the glyph's background

    //Don't draw blank characters
    if (glyph.charCode == 0 || glyph.charCode == " ".codeUnitAt(0)) return;

    _context.fillStyle =
        glyph.fore.cssString; //Set context's color to glyph's fore color
    _context.fillText(
        String.fromCharCode(glyph.charCode),
        (x * _font.charWidth + _font.x) * _scale,
        (y * _font.lineHeight + _font.y) * _scale);
  }

  void render() {
    for (int y = 0; y < rows; y++) {
      for (int x = 0; x < cols; x++) {
        Glyph glyph = _changedGlyphs.get(x, y);

        if (glyph == null) continue;

        _renderGlyph(x, y, glyph);

        _changedGlyphs.set(x, y, null);
        _glyphs.set(x, y, glyph);
      }
    }
  }

  Vector pixelToPosition(int x, int y) {
    //Map pos to position relative to canvas
    x -= _canvas.getBoundingClientRect().left;
    y -= _canvas.getBoundingClientRect().top;

    //Not in canvas
    if (x < 0 || y < 0) return null;

    int row = y ~/ _font.lineHeight * _scale;
    int col = x ~/ _font.charWidth * _scale;

    return Vector(col, row);
  }
}

==> termilary-0.2.2.tar.gz: lib/src/canvas_terminal.dart <==
import "package:utilary/utilary.dart";
import "glyph.dart";
import "terminal.dart";
import "font.dart";
import "dart:html";

class CanvasTerminal extends Terminal{
  final Array2D<Glyph> _glyphs;
  final Array2D<Glyph> _changedGlyphs;

  final Font _font;
  final CanvasElement _canvas;
  final CanvasRenderingContext2D _context;

  final int _scale = window.devicePixelRatio.toInt();

  factory CanvasTerminal(int width, int height, Font font, [CanvasElement canvas]) {
    if (canvas == null) {
      canvas = CanvasElement();
      document.body.append(canvas);
    }

    return CanvasTerminal._(width, height, font, canvas);
  }

  CanvasTerminal._(int width, int height, this._font, this._canvas)
      : _context = _canvas.getContext("2d"),
        _glyphs = Array2D(height, width),
        _changedGlyphs = Array2D(height, width, Glyph.clear),
        super(width, height) {
    int canvasWidth = cols * _font.charWidth;
    int canvasHeight = rows * _font.lineHeight;

    _canvas.width = canvasWidth * _scale;
    _canvas.height = canvasHeight * _scale;
    _canvas.style.width = "${canvasWidth}px";
    _canvas.style.height = "${canvasHeight}px";

    _context.font = "${_font.size * _scale}px ${_font.family}, monospace";
  }

  Rectangle get bounds => Rectangle(0, 0, width, height);

  @override
  void setGlyph(int x, int y, Glyph glyph) {
    if (x < 0) return;
    if (x >= cols) return;
    if (y < 0) return;
    if (y >= rows) return;

    if (_glyphs.get(x, y) != glyph)
      _changedGlyphs.set(x, y, glyph);
    else
      _changedGlyphs.set(x, y, null);
  }

  void _renderGlyph(int x, int y, Glyph glyph) {
    //Fill the background
    _context.fillStyle =
        glyph.back.cssString; //Set context's fill color as glyph's backcolor
    _context.fillRect(
        x * _font.charWidth * _scale,
        y * _font.lineHeight * _scale,
        _font.charWidth * _scale,
        _font.lineHeight * _scale); //Draw the glyph's background

    //Don't draw blank characters
    if (glyph.charCode == 0 || glyph.charCode == " ".codeUnitAt(0)) return;

    _context.fillStyle =
        glyph.fore.cssString; //Set context's color to glyph's fore color
    _context.fillText(
        String.fromCharCode(glyph.charCode),
        (x * _font.charWidth + _font.x) * _scale,
        (y * _font.lineHeight + _font.y) * _scale);
  }

  void render() {
    for (int y = 0; y < rows; y++) {
      for (int x = 0; x < cols; x++) {
        Glyph glyph = _changedGlyphs.get(x, y);

        if (glyph == null) continue;

        _renderGlyph(x, y, glyph);

        _changedGlyphs.set(x, y, null);
        _glyphs.set(x, y, glyph);
      }
    }
  }

  Vector pixelToPosition(int x, int y) {
    //Map pos to position relative to canvas
    x -= _canvas.getBoundingClientRect().left;
    y -= _canvas.getBoundingClientRect().top;

    //Not in canvas
    if (x < 0 || y < 0) return null;

    int row = y ~/ _font.lineHeight * _scale;
    int col = x ~/ _font.charWidth * _scale;

    return Vector(col, row);
  }
}
